// start of file Algo358E.cpp ---------------------------------------------

char
 name[]   = "Algo358E - Tests the CSVD routine, ACM Algorithm 358",
 version[]= "C++ Version 1.0, last revised: 2006-11-21",
 author1[]= "Copyright (c) 2006 by author: Harry J. Smith,",
 author2[]= "19628 Via Monte Dr., Saratoga, CA 95070.  All rights reserved.";

char usage1[] = "Usage: Algo358E [1, 2]";

/*****************************************************************************

Singular Value Decomposition, a = u * s * Conj(Tran(v)), a is destroyed by CSVD
the diagonal matrix s is output as a vector, m must be >= n
if smaller, a should be filled with zero rows
this code is adapted from Collected Algorithms from ACM, Algorithm 358
The transformation Conj(Tran(u)) is applied to the p vectors given in columns
n, n+1, ..., n+p-1 of matrix a
See: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358.f
and: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358_prb.f

*****************************************************************************/

/// 	<summary>
///		Tests the CSVD routine, ACM Algorithm 358
///	</summary>

#include "Algo358.h"
#include <stdlib.h>	// for atoi
#include <time.h>		// for _strdate and _strtime
#include <iomanip.h>	// for setiosflags and setprecision
#define nL	'\n'    	// new line
#define dL	"\n\n"  	// double new line
typedef complex dcomplex;  			// ??
//typedef complex complex<float>;	// could not get complex float
dcomplex zz;
void TestCSVD();
void TimeStamp();
void WriteMatrix(complex [][maxA], int, int);
int ti;		// test case number, 1 or 2

// ------- initialize
void init(void)
{
   cout << setiosflags(ios::fixed)
			<< setprecision(4) << "";
	cout << nL << name << nL;		// show credits
  	cout << version << nL;
  	cout << author1 << nL;
  	cout << author2 << dL;
   cout << usage1 << dL;	  	// show usage
} // --end-- init

// The main entry point for the application Algo358E
int main(int argc, char* argv[])
{
	init();
   char tc[2] = "1";
	if (argc > 1)                	// if arguments
		strncpy(tc, argv[1], 1);	// Copy string from user input line
	ti = atoi(tc);
   if (ti != 1 && ti != 2)
   	ti = 1;
   cout << "Running test case " << ti << " of 2" << dL;
	TimeStamp();
	cout << nL << "Algo358E:" << nL;
	cout << "Test the ACM TOMS 358 Algorithm CSVD" << dL;
	TestCSVD();
	cout << "Algo358E:" << nL;
	cout << "Normal end of execution" << dL;
	TimeStamp();
	return 0;
} // main

void TestCSVD()
//test CSVD
{
	int i, j, k;
	int m = 5;
	int n = 5;
   if (ti == 2)
   	n = 4;
	complex a[maxA][maxA];
	float s[maxA];
	complex u[maxA][maxA];
	complex v[maxA][maxA];
	cout << "TestCSVD" << nL;
	cout << "Call ACM TOMS Algorithm 358 for the" << nL;
	cout << "singular value decomposition:" << nL;
	cout << "  A = U * S * V^H" << nL;
	cout << "  of an M by N complex matrix A." << nL;
	a[0][0] = complex(0.4499F, -0.1267F);	// Set the matrix
	a[0][1] = complex(-0.2361F,  0.0775F);
	a[0][2] = complex(0.5008F, -0.7799F);
	a[0][3] = complex(-0.7702F, -0.3143F);
	a[0][4] = complex(-0.5084F,  0.5621F);
	a[1][0] = complex(-0.8432F, -0.3443F);
	a[1][1] = complex(0.0186F, -0.6332F);
	a[1][2] = complex(0.3505F,  0.0166F);
	a[1][3] = complex(-0.8892F,  0.2657F);
	a[1][4] = complex(-0.5066F,  0.6005F);
	a[2][0] = complex(0.5896F,  0.2601F);
	a[2][1] = complex(0.8928F,  0.0103F);
	a[2][2] = complex(0.4350F, -0.2666F);
	a[2][3] = complex(-0.7799F, -0.5512F);
	a[2][4] = complex(-0.1022F, -0.4500F);
	a[3][0] = complex(0.3911F,  0.3234F);
	a[3][1] = complex(-0.5605F,  0.7638F);
	a[3][2] = complex(-0.2009F,  0.2707F);
	a[3][3] = complex(0.0314F, -0.4336F);
	a[3][4] = complex(-0.1046F,  0.3267F);
	a[4][0] = complex(-0.1395F, -0.1561F);
	a[4][1] = complex(0.3064F,  0.0263F);
	a[4][2] = complex(-0.0975F,  0.9019F);
	a[4][3] = complex(0.2491F,  0.5787F);
	a[4][4] = complex(0.4106F, -0.8102F);
	cout << nL << "Matrix A:" << dL;
	WriteMatrix(a, m, n);
	int p = 0;
	int nu = m;
	int nv = n;
	CSVD(a, m, n, p, nu, nv, s, u, v);
	cout << "Singular values:" << dL;
	for (i = 0; i < n; i++)
		cout << setprecision(6) << "  " << s[i] << nL;
	cout << setprecision(4) << "";
	cout << nL << "U:" << dL;
	WriteMatrix(u, m, m);
	cout << "V:" << dL;
	WriteMatrix(v, n, n);
	for (i = 0; i < m; i++)	 // Compute U * S * V^H
		for (j = 0; j < n; j++)
		{
			a[i][j] = 0.0F, 0.0F;
			for (k = 0; k < m && k < n; k++)
				a[i][j] += u[i][k] * s[k] * conj(v[j][k]);
		}
	cout << "Matrix U S V^H" << nL;
	cout << "(should equal the original A):" << dL;
	WriteMatrix(a, m, n);
} // TestCSVD

void WriteMatrix(complex a[][maxA], int m, int n)
// print an m by n complex matrix
{
	for (int i = 0; i < m; i++)
	{
   	for (int j = 0; j < n; j++)
		{
			cout << " ";
			if (real(a[i][j]) >= 0.0F)
				cout << " ";
         else
         	cout << "-";
			cout << ABS(real(a[i][j]));
			if (imag(a[i][j]) >= 0.0F)
				cout << "+";
         else
         	cout << "-";
         cout << ABS(imag(a[i][j]));
   	}
		cout << nL;
	}
	cout << nL;
} // WriteMatrix

void TimeStamp()
// printout the current HH:MM:SS as a timestamp
{
   char da[10];       // Date
	_strdate(da);
   char ti[10];       // Time
   _strtime(ti);
	cout << da << " " << ti << nL;
} // TimeStamp

// end of file Algo358E.cpp -----------------------------------------------

